{-# LANGUAGE DataKinds #-}
module Test.Random
    ( tests
    ) where

import           Conduit
import           Control.Monad.ST
import           Data.List
import           Data.List.Ordered (nubSort)
import           System.IO.Unsafe
import           Test.Tasty
import           Test.Tasty.HUnit
import           Test.Utils

import           IGraph
import           IGraph.Internal
import           IGraph.Random
import qualified IGraph.Mutable    as GM

tests :: TestTree
tests = testGroup "Random tests"
    [ testCase "withSeed works" withSeedWorks
    ]

withSeedWorks :: Assertion
withSeedWorks = do
  stockRng <- igraphRngDefault
  res0 <- igraphRngGetInteger stockRng 1 10
  -- Starting from the same seed, but with two different generators
  -- must yield the same result, which is not the one from the stock RGN
  res1 <- withSeed 42 $ \(Gen gen) -> igraphRngGetInteger gen 1 10
  assertBool "withSeedWorks" $ res1 >= 1
  assertBool "withSeedWorks" $ res1 <= 10

  res2 <- withSeed 42 $ \(Gen gen) -> igraphRngGetInteger gen 1 10
  assertBool "res1 not equal res2" $ res1 == res2
  assertBool "res0 not unequal"    $ res0 /= res1

